# -*- coding: UTF-8 -*-
# Author:  Werner F. Bruhin
# Purpose: Application constants
# Created: 06/04/2012

import wx

# language domain
langDomain = "videomass"
# languages you want to support
supLang = {"en": wx.LANGUAGE_ENGLISH,
           "it": wx.LANGUAGE_ITALIAN,
           "fr": wx.LANGUAGE_FRENCH,
           "de": wx.LANGUAGE_GERMAN,
           "pl": wx.LANGUAGE_POLISH,
           "es": wx.LANGUAGE_SPANISH,
           }
